#include "common.h"

#include <stdio.h> 
#include <stdlib.h>
#include <time.h>
#include <stdint.h>
#include <stdbool.h>

#define DEBUG_LEVEL 0

long long getTimeInMs(void)
{
    struct timespec spec;
    clock_gettime(CLOCK_REALTIME, &spec);
    long long seconds = spec.tv_sec;
    long long nanoSeconds = spec.tv_nsec;
    long long milliSeconds = seconds * 1000 + (nanoSeconds / 1000000);
    return milliSeconds;
}

long long getTimeInNs(void)
{
    struct timespec spec;
    clock_gettime(CLOCK_REALTIME, &spec);
    long long seconds = spec.tv_sec;
    long long nanoSeconds = spec.tv_nsec;
    long long ret = seconds * 1000000000 + nanoSeconds;
    //long long milliSeconds = seconds * 1000 + (nanoSeconds / 1000000);
    return ret;
}


void sleepForMs(long long delayInMs)
{
    const long long NS_PER_MS = 1000 * 1000;
    const long long NS_PER_SECOND = 1000000000;
    long long delayNs = delayInMs * NS_PER_MS;
    int seconds = delayNs / NS_PER_SECOND;
    int nanoseconds = delayNs % NS_PER_SECOND;
    struct timespec reqDelay = {seconds, nanoseconds};
    nanosleep(&reqDelay, (struct timespec *) NULL);
}


void runCommand(char* command)
{
    // Execute the shell command (output into pipe)
    FILE *pipe = popen(command, "r");
    // Ignore output of the command; but consume it
    // so we don't get an error when closing the pipe.
    char buffer[1024];
    while (!feof(pipe) && !ferror(pipe)) {
        if (fgets(buffer, sizeof(buffer), pipe) == NULL)
            break;
            // printf("--> %s", buffer); // Uncomment for debugging
    }
    // Get the exit code from the pipe; non-zero is an error:
    int exitCode = WEXITSTATUS(pclose(pipe));
    if (exitCode != 0) {
        perror("Unable to execute command:");
        printf(" command: %s\n", command);
        printf(" exit code: %d\n", exitCode);
    }
}

void writeToFile(char* fileName, char* msgToWrite)
{
    // Use fopen() to open the file for write access.
    FILE *pFile = fopen(fileName, "w");
    if (pFile == NULL) {
        printf("ERROR: Unable to open file in writeToFile.\n");
        printf("Filename given: (%s)\n", fileName);
        exit(1);
    }
    // Write to data to the file using fprintf():
    fprintf(pFile, "%s", msgToWrite);
    // Close the file using fclose():
    fclose(pFile);
}

int readFromFile(char* fileName)
{
    FILE *pFile = fopen(fileName, "r");
    if (pFile == NULL) {
        printf("ERROR: Unable to open file (%s) for read\n", fileName);
        exit(-1);
    }
    // Read string (line)
    const int MAX_LENGTH = 1024;
    char buff[MAX_LENGTH];
    fgets(buff, MAX_LENGTH, pFile);
    // Close
    fclose(pFile);

    #if DEBUG_LEVEL >= 1
        printf("Read from file: %s\n", buff);
    #endif

    //convert to int and return 
    long fromStr = atol(buff);
    return (int)fromStr;
}

void initializeGPIoPin(unsigned busNumber, unsigned bbgPinNumber, unsigned linuxFilePinNumber, xPut inOrOut)
{
    if(busNumber != 8 && busNumber != 9) {
        printf("Failed to initialize a GPIO port. Bus number given: %d\n", busNumber);
        return;
    }
    if(bbgPinNumber > 46) {
        printf("Failed to initialize a GPIO port. BBG pin number given: %d\n", bbgPinNumber);
        return;
    }
    char configCommand[24];
    snprintf(configCommand, 23, "config-pin p%d.%d gpio", busNumber, bbgPinNumber);
    runCommand(configCommand);

    if(linuxFilePinNumber > 120) {
        printf("Failed to initialize a GPIO port. Linux pin number given: %d\n", linuxFilePinNumber);
        return;
    }
    char directionFilePath[35];
    snprintf(directionFilePath, 34, "/sys/class/gpio/gpio%d/direction", linuxFilePinNumber);
    if(inOrOut == in) {
        writeToFile(directionFilePath, "in");
    }
    else {
        writeToFile(directionFilePath, "out");
    }
}