#include "tempSensor.h"
#include "common.h"

#include <stdlib.h> 
#include <stdio.h> 
#include <stdbool.h>

static void initializeExternals(void);
static void cleanup(void);
static void blockUntilSensorReady(void);

//starts up the sensor, then reads it every ~2s 
int main(void)
{
    printf("Initializing...\n");
    initializeExternals();
    blockUntilSensorReady();
    printf("Use CTRL+C to quit.\n");
    while(true)
    {
        float temperatureRead = tempSensor_getCurrentTemp();
        float percHumidityRead = tempSensor_getCurrentPercentHumidity();
        printf("Read values: temperature = %6.2f, %%humidity = %6.2f\n", temperatureRead, percHumidityRead);
        sleepForMs(2000);
    }
    cleanup();
    sleepForMs(500);
    printf("Cleaned up, thanks for using!\n");
    return 0;
}

static void initializeExternals(void)
{
    tempSensor_startSampling();
}

static void cleanup(void)
{
    tempSensor_stopSampling();
}

static void blockUntilSensorReady(void)
{
    while(!tempSensor_readyToReadValues())
    {
        sleepForMs(500);
    }
}