/**
 * This module on how to use the RHT03 temperature and humidity sensor was written for 
 * ENSC351, featuring ideas and code distributed in class. 
 * This module uses the following pins as I/O for the BeagleBone Green's (BBG) PRU I/O 
 *     p9_27 (linux 115): indicator LED for debugging 
 *     p9_28 (linux 113): button for debugging 
 *     p9_31 (linux 110): PRU output to signal the sensor 
 *     p9_29 (linux 111): PRU input to read from the sensor 
 * All of these pins must be configured upon every login to the BBG; tempSensor.c will 
 * automatically configure these pins upon run. 

 * Note that the sensor updates slowly; if moved from a warm place to a cold one, it takes time to 
 * reflect that in the read values. 
*/

#ifndef TEMPSENSOR_H
#define TEMPSENSOR_H 

#include <stdbool.h>

/**
 * Starts up the sampling thread. 
 * Wait until readyToReadValues returns true before attempting to read. 
*/
void tempSensor_startSampling(void);
/**
 * Terminates the sensor thread and cleans up memory used. 
*/
void tempSensor_stopSampling(void);

/**
 * Returns temperature read by sensor 
*/
float tempSensor_getCurrentTemp(void);
/**
 * Returns %humidity read by sensor 
*/
float tempSensor_getCurrentPercentHumidity(void);

/**
 * Returns whether sensor has successfully read values yet.
 * Attempting to get values from sensor before this is true will 
 * always return 0 and 0 for temp and %hum
*/
bool tempSensor_readyToReadValues(void);

#endif 